@@ -6,7 +6,6 @@
 #' These functions are untested on Windows and macOS and require:
 #'  * `bash`
 #'  * `docker`
-#'  * `docker-compose`
 #'
 #' @param user Character vector for user name (defaults to "rods" admin)
 #' @param pass Character vector for password (defaults to "rods" admin password)
@@ -38,15 +37,10 @@
 use_irods_demo <- function(user = character(), pass = character(),
                            recreate = FALSE, verbose = TRUE) {
 
-  # check if Docker is installed and can be accessed without sudo rights
-  docker_version <- system("docker --version", ignore.stdout = !verbose,
-                           ignore.stderr = !verbose)
-  if (Sys.which("bash") == "" ||
-      Sys.which("docker") == ""  ||
-      docker_version == "" ||
-      Sys.which("docker-compose") == "") {
+  # is Docker installed
+  if (!check_docker()) {
     stop(
-      "Bash and Docker with the docker-compose plugin are required. \n",
+      "Bash and Docker are required. \n",
       "Install bash and docker to commence. Alternatively, sudo rights \n",
       "are required for Docker: please check: \n",
       "https://docs.docker.com/engine/install/linux-postinstall/",
@@ -55,20 +49,12 @@ use_irods_demo <- function(user = character(), pass = character(),
   }
 
   # do irods_demo images exist on this machine?
-  irods_images <- system2(
-    system.file(package = "rirods", "shell_scripts", "docker-images.sh"),
-    stdout = TRUE,
-    stderr = FALSE
-  )
-
   resp_user <- TRUE
-  if (length(irods_images) == 0 ||
-      !all(grepl(paste0(irods_images_ref, collapse = "|"), irods_images))
-      ) {
-    message("\nThe iRODS demo Docker containers are not built on this system. \n")
+  if (!check_irods_images()) {
+    message("\nThe iRODS demo Docker images are not available on this system. \n")
     if (interactive()) {
       resp_user <-
-        utils::askYesNo("Would you like it to be built?", default = FALSE)
+        utils::askYesNo("Would you like it to be pull the iRODS demo Docker images?", default = FALSE)
     }
   }
 
@@ -82,35 +68,43 @@ use_irods_demo <- function(user = character(), pass = character(),
   if (length(user) != 0 && length(pass) != 0) {
     system2(
       system.file(package = "rirods", "shell_scripts", "iadmin-docker-icommand.sh"),
-      c(user, pass),
+      Map(shQuote, c(user, pass)),
       stdout = FALSE,
       stderr = FALSE
-     )
+    )
   } else {
     user <- pass <- "rods"
   }
 
   # Sometimes it just does not want to start right. This will perform a dry run
   # and restart the process again. This usually does the trick.
-  dry_run_irods(user, pass, recreate)
+  dry_run_irods(
+    user,
+    pass,
+    .irods_host,
+    paste0("/tempZone/home/", user),
+    verbose
+  )
 
-  if (isTRUE(verbose)) {
-    message(
-      "\n",
-      "Do the following to connect with the iRODS demo server: \n",
-      "create_irods(\"http://localhost/irods-rest/0.9.3\", \"/tempZone/home\") \n",
-      "iauth(\"", user, "\", \"", pass, "\")"
-    )
-  }
+  message(
+    "\n",
+    "Do the following to connect with the iRODS demo server: \n",
+    "create_irods(\"", .irods_host, "\") \n",
+    "iauth(\"", user, "\", \"", pass, "\")"
+  )
 
-  invisible()
+  invisible(NULL)
 }
 #' @rdname use_irods_demo
 #'
 #' @export
-stop_irods_demo <- function() {
-  system(paste0("cd ", path_to_demo(), " ; docker-compose down"))
-  invisible()
+stop_irods_demo <- function(verbose = TRUE) {
+  system(
+    paste0("cd ", path_to_demo(), " ; docker compose down"),
+    ignore.stdout = !verbose,
+    ignore.stderr = !verbose
+  )
+  invisible(NULL)
 }
 
 #' Predicate for iRODS Demonstration Service State
@@ -124,52 +118,54 @@ stop_irods_demo <- function() {
 #' @examples
 #' is_irods_demo_running()
 is_irods_demo_running <- function(...) {
-
-  # check for client-icommand is not required (only needed for demo itself)
-  irods_container_ref <- paste0(irods_images_ref, "_1")
-  irods_container_ref <-
-    irods_container_ref[irods_container_ref != "irods_demo_irods-client-icommands_1"]
-  is_irods_demo_running_ <- function(x) {
-    system2(
-      system.file(package = "rirods", "shell_scripts", "docker-containers.sh"),
-      x,
-      stderr = NULL
-    )
+  # first check if Docker exist
+  if (!check_docker(FALSE)) {
+    return(FALSE)
   }
+  # then check if images exist
+  if (!check_irods_images()) {
+    return(FALSE)
+  }
+  # check for client-icommand is not required (only needed for demo itself)
+  ref <- irods_containers_ref()
+  ref <- ref[ref != "irods-demo-irods-client-icommands-1"]
   irods_containers_state <-
-    vapply(
-      irods_container_ref,
-      is_irods_demo_running_,
-      integer(1)
-    )
+    vapply(ref, is_irods_demo_running_, integer(1))
 
   if (sum(irods_containers_state) == 0) TRUE else FALSE
 }
 
+# this seems not work on Windows
+is_irods_demo_running_ <- function(x) {
+  system2(
+    system.file(package = "rirods", "shell_scripts", "docker-containers.sh"),
+    shQuote(x),
+    stderr = NULL
+  )
+}
+
 #' Remove Docker images from system
 #' @keywords internal
 #' @return Invisible
+#' @noRd
 remove_docker_images <- function() {
   if (is_irods_demo_running()) {
     stop("Docker containers are still running. Stop them with ",
          "`stop_irods_demo()` and proceed", call. = FALSE)
   }
-  bsh <-
-    system.file(package = "rirods", "shell_scripts", "docker-images-remove.sh")
-  Map(
+  invisible(Map(
     function(x) {
-      system2(bsh, x, stderr = NULL)
+      system(paste("docker rmi", x), ignore.stderr = TRUE)
     },
-    irods_images_ref
-  )
-  invisible()
+    irods_images_ref("id")
+  ))
 }
 
 start_irods <- function(verbose, recreate = TRUE) {
   if (isTRUE(recreate)) {
-    cmd <- " ; docker-compose up -d --force-recreate nginx-reverse-proxy"
+    cmd <- " ; docker compose up -d --force-recreate nginx-reverse-proxy irods-client-http-api irods-client-icommands"
   } else {
-    cmd <- " ; docker-compose up -d nginx-reverse-proxy"
+    cmd <- " ; docker compose up -d nginx-reverse-proxy irods-client-http-api irods-client-icommands"
   }
   system(
     paste0("cd ", path_to_demo(), cmd),
@@ -181,37 +177,106 @@ start_irods <- function(verbose, recreate = TRUE) {
 path_to_demo <- function() system.file("irods_demo", package = "rirods")
 
 # perform dry run to see if iRODS can be used
-dry_run_irods <- function(user, pass, recreate) {
-  tryCatch(
-    {local_create_irods(); iauth(user, pass)},
-    error = function(err)  {
-      if (isFALSE(recreate)) {
-        message(
-          "\nThere seems to be a problem with the iRODS demo ",
-          "server. \nThe problem might be solved by rebooting the server. ",
-          "\nThis action will destroy all content on the server!\n"
-        )
-        recreate <- utils::askYesNo("Can I reboot the server?", default = FALSE)
-      }
+dry_run_irods <- function(user, pass, host, lpath, verbose, user_input = FALSE) {
 
-      if (isTRUE(recreate)) {
-        message("\nThis may take a while!\n")
-        start_irods(recreate)
-        use_irods_demo(user, pass, recreate, verbose = FALSE)
-      } else {
-        stop("The iRODS server could not be started!", call. = FALSE)
-      }
+  irods_server_status <- is_irods_server_operational(user, pass, host, lpath)
+
+  while (!irods_server_status) {
+    if (isFALSE(user_input)) {
+      message(
+        "\nThere seems to be a problem with the iRODS demo ",
+        "server. \nThe problem might be solved by rebooting the server. ",
+        "\nThis action will destroy all content on the server!\n"
+      )
+      user_input <- utils::askYesNo("Can I reboot the server?", default = FALSE)
     }
-  )
+    if (isTRUE(user_input)) {
+      if (verbose) message("\nRecreating iRODS demo. This may take a while!\n")
+      start_irods(verbose, recreate = TRUE)
+    } else{
+      stop("The iRODS server could not be started!", call. = FALSE)
+    }
+
+    irods_server_status <- is_irods_server_operational(user, pass, host, lpath)
+  }
+}
+
+is_irods_server_operational <- function(user, pass, host, lpath) {
+  Sys.sleep(3) # requires some time to stand up
+  is_irods_server_running_correct() &&
+    is_http_server_running_correct(user, pass, host, lpath)
+}
+
+is_http_server_running_correct <- function(user, pass, host, lpath) {
+  system2(
+    system.file(package = "rirods", "shell_scripts", "dry-run-irods-curl.sh"),
+    Map(shQuote, c(user, pass, host, lpath)),
+    stdout = FALSE,
+    stderr = FALSE
+  ) == 0
+}
+
+is_irods_server_running_correct <- function() {
+  system2(
+    system.file(package = "rirods", "shell_scripts", "dry-run-irods-icommands.sh"),
+    stdout = FALSE,
+    stderr = FALSE
+  ) == 0
 }
 
 # look up table for irods_demo images
-irods_images_ref <- c(
-  "irods_demo_irods-catalog",
-  "irods_demo_irods-catalog-provider",
-  "irods_demo_irods-client-icommands",
-  "irods_demo_irods-client-rest-cpp",
-  "irods_demo_nginx-reverse-proxy"
+irods_images_ref <- function(filter_images = "name") {
+  if (filter_images == "id") {
+    dkr_format <- "{{.ID}}"
+  } else if (filter_images == "name") {
+    dkr_format <- "{{.Repository}}"
+  }
+  dkr_args <- c("irods-demo*", "irods/*", dkr_format)
+  dkr_args <- Map(shQuote, dkr_args)
+  cmd <- system.file(package = "rirods", "shell_scripts", "docker-images.sh")
+  # this does not work on all Windows OS
+  imgs <- try(system2(cmd, args = dkr_args, stdout = TRUE), silent = TRUE)
+  if (inherits(imgs, "try-error")) {
+    return(NULL)
+  } else {
+    return(imgs)
+  }
+}
+
+# are the images available on this system
+check_irods_images <- function() {
+  if (length(irods_images_ref()) == 0) {
+    check_images_result <- FALSE
+  } else {
+    check_images_result <-
+      all(grepl(paste0(irods_images, collapse = "|"), irods_images_ref()))
+  }
+  check_images_result
+}
+
+# does Docker exist
+check_docker <- function(verbose = TRUE) {
+  # check if Docker is installed and can be accessed without sudo rights
+  docker_version <- system("docker --version", ignore.stdout = !verbose,
+                           ignore.stderr = !verbose)
+  !(Sys.which("bash") == "" || Sys.which("docker") == "" || docker_version == "")
+}
+
+irods_containers_ref <- function() {
+  irods_demo_yml <- system.file("irods_demo", "docker-compose.yml", package = "rirods")
+  irods_demo_file <- readLines(irods_demo_yml)
+  irods_images_ref <- grep("^\\s{4}[[:graph:]]*?:$", irods_demo_file)
+  irods_images <- irods_demo_file[irods_images_ref]
+  paste0("irods-demo-", trimws(gsub( ":$", "", irods_images)), "-1")
+}
+
+irods_images <- c(
+  "irods-demo-irods-catalog",
+  "irods-demo-irods-catalog-provider",
+  "irods-demo-irods-client-icommands",
+  "irods-demo-irods-client-rest-cpp",
+  "irods-demo-nginx-reverse-proxy",
+  "irods/irods_http_api"
 )
 
 #' Launch iRODS from Alternative Directory
@@ -220,20 +285,18 @@ irods_images_ref <- c(
 #' package source files.
 #'
 #' @param host Hostname of the iRODS server. Defaults to
-#'  "http://localhost/irods-rest/0.9.3".
-#' @param zone_path Zone path of the iRODS server. Defaults to "/tempZone/home".
+#'  ""http://localhost:9001/irods-http-api/0.1.0".
 #' @param dir The directory to use. Default is a temporary directory.
 #' @param env Attach exit handlers to this environment. Defaults to the
 #'  parent frame (accessed through [parent.frame()]).
 #'
 #' @return Invisibly returns the original directory.
 #' @keywords internal
-#'
+#' @noRd
 local_create_irods <- function(
-    host = NULL,
-    zone_path = NULL,
-    dir = tempdir(),
-    env = parent.frame()
+  host = NULL,
+  dir = tempdir(),
+  env = parent.frame()
 ) {
 
   # default host
@@ -242,17 +305,7 @@ local_create_irods <- function(
       host <-
         httr2::secret_decrypt(Sys.getenv("DEV_HOST_IRODS"), "DEV_KEY_IRODS")
     } else {
-      host <- "http://localhost/irods-rest/0.9.3"
-    }
-  }
-
-  # defaults path
-  if (is.null(zone_path)) {
-    if (Sys.getenv("DEV_KEY_IROD") != "") {
-      zone_path <-
-        httr2::secret_decrypt(Sys.getenv("DEV_ZONE_PATH_IRODS"), "DEV_KEY_IRODS")
-    } else {
-      zone_path <- "/tempZone/home"
+      host <- .irods_host
     }
   }
 
@@ -264,7 +317,7 @@ local_create_irods <- function(
   withr::defer(setwd(old_dir), envir = env)
 
   # switch to new iRODS project
-  create_irods(host, zone_path, overwrite = TRUE)
+  create_irods(host, overwrite = TRUE)
   withr::defer(unlink(path_to_irods_conf()), envir = env)
 
   invisible(dir)
