@@ -8,7 +8,7 @@
 #'
 #' @param logical_path Path to the data object or collection (or name of the user).
 #' @param entity_type Type of item to add metadata to or remove it from.
-#'   Options are 'data_object', 'collection' and 'user'.
+#'   Options are 'data_object', 'collection' and 'user'.  Deprecated.
 #' @param operations List of named lists or [data.frame] representing operations.
 #'   The valid components of each of these lists or vectors are:
 #'   - `operation`, with values 'add' or 'remove', depending on whether the AVU
@@ -16,6 +16,7 @@
 #'   - `attribute`, with the name of the AVU (required).
 #'   - `value`, with the value of the AVU (required).
 #'   - `units`, with the unit of the AVU (optional).
+#' @param admin Whether to grant admin rights. Defaults to `FALSE`.
 #' @param verbose Whether information should be printed about the HTTP request
 #'  and response. Defaults to `FALSE`.
 #'
@@ -32,9 +33,12 @@
 #'
 #' # demonstration server (requires Bash, Docker and Docker-compose)
 #' # use_irods_demo()
-#'
+#' \dontshow{
+#' .old_config_dir <- Sys.getenv("R_USER_CONFIG_DIR")
+#' Sys.setenv("R_USER_CONFIG_DIR" = tempdir())
+#' }
 #' # connect project to server
-#' create_irods("http://localhost/irods-rest/0.9.3", "/tempZone/home")
+#' \Sexpr[stage=build, results=rd]{paste0("create_irods(\"", rirods:::.irods_host, "\")")}
 #'
 #' # authentication
 #' iauth("rods", "rods")
@@ -51,7 +55,6 @@
 #' # add some metadata
 #' imeta(
 #'   "foo.rds",
-#'   "data_object",
 #'    operations =
 #'     list(
 #'      list(operation = "add", attribute = "foo", value = "bar", units = "baz")
@@ -61,7 +64,6 @@
 #' # `operations` can contain multiple tags supplied as a `data.frame`
 #' imeta(
 #'   "foo.rds",
-#'   "data_object",
 #'   operations = data.frame(
 #'     operation = c("add", "add"),
 #'     attribute = c("foo2", "foo3"),
@@ -73,7 +75,6 @@
 #' # or again as a list of lists
 #' imeta(
 #'   "foo.rds",
-#'   "data_object",
 #'   operations = list(
 #'     list(operation = "add", attribute = "foo4", value = "bar4", units = "baz4"),
 #'     list(operation = "add", attribute = "foo5", value = "bar5", units = "baz5")
@@ -83,7 +84,6 @@
 #' # list of lists are useful as AVUs don't have to contain units
 #' imeta(
 #'   "foo.rds",
-#'   "data_object",
 #'   operations = list(
 #'     list(operation = "add", attribute = "foo6", value = "bar6"),
 #'     list(operation = "add", attribute = "foo7", value = "bar7", units = "baz7")
@@ -95,16 +95,23 @@
 #'
 #' # delete object
 #' irm("foo.rds", force = TRUE)
-#'
+#' \dontshow{
+#' Sys.setenv("R_USER_CONFIG_DIR" = .old_config_dir)
+#' }
 imeta <- function(
     logical_path,
     entity_type = c("data_object", "collection", "user"),
     operations = list(),
+    admin = FALSE,
     verbose = FALSE
 ) {
 
+  # deprecate arguments
+  if (!missing("entity_type"))
+    warning("Argument `entity_type` is deprecated")
+
   # expand logical path to absolute logical path
-  logical_path <- get_absolute_lpath(logical_path, open = "read")
+  logical_path <- get_absolute_lpath(logical_path)
 
   # define entity type
   entity_type <- match.arg(entity_type)
@@ -142,15 +149,22 @@ imeta <- function(
   }
 
   # data to be converted to json for body (double operation list important for boxing)
-  json <- list(
-    entity_name = logical_path,
-    entity_type = entity_type,
-    operations = operations
+  args <- list(
+    op = "modify_metadata",
+    lpath = logical_path,
+    operations = jsonlite::toJSON(operations, auto_unbox = TRUE),
+    admin = as.integer(admin)
   )
 
-  # http call
-  resp <- irods_rest_call("metadata", "POST", args = list(), verbose, json)
-
+  if (is_collection(logical_path)) {
+    resp <- irods_http_call("collections", "POST", args, verbose = FALSE) |>
+            httr2::req_perform()
+  } else if (is_object(logical_path)) {
+    resp <- irods_http_call("data-objects", "POST", args, verbose = FALSE) |>
+            httr2::req_perform()
+  } else {
+    stop("Unkown operation", call. = FALSE)
+  }
   invisible(resp)
 }
 
@@ -171,35 +185,42 @@ imeta <- function(
 #' @param verbose Whether information should be printed about the HTTP request
 #'  and response. Defaults to `FALSE`.
 #'
-#' @return A dataframe with one row per result and one column per requested attribute,
-#'   with "size" and "time" columns parsed to the right type.
+#' @return A dataframe with one row per result and one column per requested
+#'  attribute, with "size" and "time" columns parsed to the right type.
 #' @seealso [imeta()]
 #'
 #' @references
 #'  https://docs.irods.org/master/icommands/user/#iquest
 #'
-#' Use SQL-like expressions to query data objects and collections based on different properties.
+#' Use SQL-like expressions to query data objects and collections based on
+#'  different properties.
 #'
 #' @param query GeneralQuery for searching the iCAT database.
-#' @param limit Maximum number of rows to return. Defaults to 100.
-#' @param offset Number of rows to skip for paging. Defaults to 0.
+#' @param limit Maximum number of rows to return.
+#' @param offset Number of rows to skip for paging. Deprecated.
 #' @param type Type of query: 'general' (the default) or 'specific'.
-#' @param case_sensitive Whether the string matching in the query is case sensitive.
-#'   Defaults to `TRUE`.
-#' @param distinct Whether only distinct rows should be listed. Defaults to `TRUE`.
-#' @param verbose Whether information should be printed about the HTTP request and response.
+#' @param case_sensitive Whether the string matching in the query is case
+#'  sensitive. Defaults to `TRUE`.
+#' @param distinct Whether only distinct rows should be listed. Defaults to
+#'  `TRUE`.
+#' @param parser Which parser to use: genquery1 or genquery2. Defaults to
+#'  genquery1.
+#' @param sql_only Whether to dry-run query and return SQL syntax query as
+#'  return. Defaults to `FALSE`. Needs [Genquery2](https://github.com/irods/irods_api_plugin_genquery2/?#api-plugin).
+#' @param verbose Whether information should be printed about the HTTP request
+#'  and response.
 #'
 #' @return Invisibly, the HTTP response.
 #' @export
 #'
 #' @examplesIf is_irods_demo_running()
 #' is_irods_demo_running()
-#'
-#' # demonstration server (requires Bash, Docker and Docker-compose)
-#' # use_irods_demo()
-#'
+#' \dontshow{
+#' .old_config_dir <- Sys.getenv("R_USER_CONFIG_DIR")
+#' Sys.setenv("R_USER_CONFIG_DIR" = tempdir())
+#' }
 #' # connect project to server
-#' create_irods("http://localhost/irods-rest/0.9.3", "/tempZone/home")
+#' \Sexpr[stage=build, results=rd]{paste0("create_irods(\"", rirods:::.irods_host, "\")")}
 #'
 #' # authentication
 #' iauth("rods", "rods")
@@ -213,7 +234,6 @@ imeta <- function(
 #' # add metadata
 #' imeta(
 #'   "foo.rds",
-#'   "data_object",
 #'   operations =
 #'     list(
 #'       list(operation = "add", attribute = "bar", value = "baz")
@@ -225,7 +245,9 @@ imeta <- function(
 #'
 #' # delete object
 #' irm("foo.rds", force = TRUE)
-#'
+#' \dontshow{
+#' Sys.setenv("R_USER_CONFIG_DIR" = .old_config_dir)
+#' }
 iquery <- function(
     query,
     limit = 100,
@@ -233,29 +255,39 @@ iquery <- function(
     type = c('general', 'specific'),
     case_sensitive = TRUE,
     distinct = TRUE,
+    parser = c("genquery1", "genquery2"),
+    sql_only = FALSE,
     verbose = FALSE
   ) {
-  type <- match.arg(type)
+
+  parser <- match.arg(parser)
+
+  # deprecate arguments
+  if (!missing("type"))
+    warning("Argument `type` is deprecated")
 
   # flags to curl call
   args <- list(
-    limit = limit,
+    op = "execute_genquery",
+    query = query,
     offset = offset,
-    type = type,
+    count = find_irods_file("max_number_of_rows_per_catalog_query"),
     `case-sensitive` = as.integer(case_sensitive),
     distinct = as.integer(distinct),
-    query = query
+    parser = parser,
+    `sql-only` = as.integer(sql_only),
+    zone = find_irods_file("irods_zone")
   )
 
-  # http call
-  resp <- irods_rest_call("query", "GET", args, verbose)
+  resp <- irods_http_call("query", "GET", args, verbose) |>
+    httr2::req_perform()
 
   # response
   out <- httr2::resp_body_json(
     resp,
     check_type = FALSE,
     simplifyVector = TRUE
-  )$`_embedded`
+  )$rows
 
   try(
     {
@@ -281,5 +313,46 @@ iquery <- function(
     silent = TRUE
   )
 
-  out
+  limit_maximum_number_of_rows_catalog(out, limit)
+}
+
+data_object_metadata <- function(lpath, x = NULL) {
+  if (!is.null(x)) {
+    paste0("select META_DATA_ATTR_NAME, ",
+          "META_DATA_ATTR_VALUE, ",
+          "META_DATA_ATTR_UNITS where ",
+          "COLL_NAME = '", lpath,
+            "' and DATA_NAME = '", x, "'")
+  } else {
+    paste0("select COLL_NAME, ",
+           "DATA_NAME, ",
+           "META_DATA_ATTR_NAME, ",
+           "META_DATA_ATTR_VALUE, ",
+           "META_DATA_ATTR_UNITS where COLL_NAME = '",
+           lpath, "'")
+    }
+}
+
+collection_metadata <- function(lpath, recurse = FALSE) {
+  if (isTRUE(recurse)) {
+    paste0("select COLL_NAME, ",
+           "META_COLL_ATTR_NAME, ",
+           "META_COLL_ATTR_VALUE, ",
+           "META_COLL_ATTR_UNITS where COLL_PARENT_NAME = '",
+           lpath, "'")
+  } else {
+    paste0("select META_COLL_ATTR_NAME, ",
+           "META_COLL_ATTR_VALUE, ",
+           "META_COLL_ATTR_UNITS where COLL_NAME = '",
+           lpath, "'")
+  }
+}
+
+is_data_object_metadata <- function(lpath, x) {
+  paste0("select COLL_ID where COLL_NAME = '", lpath, "'",
+         "and DATA_NAME = '", x, "'")
+}
+
+is_collection_metadata <- function(lpath) {
+  paste0("select COLL_ID where COLL_NAME = '", lpath, "'")
 }
